#include "serverWrapper.h"
static void extractClientAddrPort(struct sockaddr* clientsockaddr, 
                                  char* addrstr, 
                                  unsigned short* port);

int 
GetAddrInfo(char* addr, 
            char* port, 
            struct addrinfo* hints, 
            struct addrinfo** result) 
{
    int status = getaddrinfo((const char*)addr, 
                             (const char*)port, 
                             (const struct addrinfo*)hints,
                             result);
    if(status != 0) {
        fprintf(stderr, "getaddrinfo(): %s\n", gai_strerror(status));
        exit(status);
    }
    return 0;
}

int
loopNewSocket(int domain, int type, int protocol) {
    int sockfd = socket(domain, type, protocol);
    if(sockfd == -1) {
        int tmp = errno;
        fprintf(stderr, "socket(): %s\n", strerror(tmp));
        return -1;
    }
    return sockfd;
}

int
loopBind(int sockfd, struct sockaddr* addr, socklen_t len) {
    int status = bind(sockfd, (const struct sockaddr*)addr, len);
    if(status == -1) {
        int tmp = errno;
        fprintf(stderr, "bind(): %s\n", strerror(tmp));
        close(sockfd);
        return -1;
    }
    return 0;
}

int
loopListen(int sockfd, int backlog) {
    int status = listen(sockfd, backlog);
    if(status == -1) {
        int tmp = errno;
        fprintf(stderr, "listen(): %s\n", strerror(tmp));
        close(sockfd);
        return -1;
    }
    return 0;
}

int
loopSetSockOpt(int sockfd, int level, int optname, 
           void* optval, socklen_t optlen)
{
    int status = setsockopt(sockfd, level, optname,
                            (const void*)optval, optlen);
    if(status == -1) {
        int tmp = errno;
        fprintf(stderr, "setsockopt(): %s\n", strerror(tmp));
        close(sockfd);
        return -1;
    }
    return 0;
}

int
loopAccept(int sockfd, 
           struct sockaddr* clientsockaddr, 
           socklen_t* clientsockaddrlen)
{
    int newsockfd = accept(sockfd, clientsockaddr, clientsockaddrlen);
    if(newsockfd == -1)
        return -1;
    else {
        char addrstr[INET6_ADDRSTRLEN];
        unsigned short port;
        extractClientAddrPort(clientsockaddr, addrstr, &port);
        fprintf(stdout, "accept(): connection from %s:%d\n", addrstr, port);
        return newsockfd;
    }
}

static void
extractClientAddrPort(struct sockaddr* clientsockaddr, 
                      char* addrstr, 
                      unsigned short* port)
{
    if(clientsockaddr->sa_family == AF_INET) {
        struct sockaddr_in* addr = (struct sockaddr_in*)clientsockaddr;
        inet_ntop(AF_INET, (const void*)&(addr->sin_addr), addrstr, INET6_ADDRSTRLEN);
        *port = ntohs(addr->sin_port);
    }
    else {
        struct sockaddr_in6* addr = (struct sockaddr_in6*)clientsockaddr;
        inet_ntop(AF_INET6, (const void*)&(addr->sin6_addr), addrstr, INET6_ADDRSTRLEN);
        *port = ntohs(addr->sin6_port);
    }
    return;
}

ssize_t
loopSend(int sockfd, void* buff, size_t buffsize, int flags) {
    ssize_t sentSize = send(sockfd, (const void*)buff, buffsize, flags);
    if(sentSize == -1) {
        int tmp = errno;
        fprintf(stderr, "send(): %s\n", strerror(tmp));
        return -1;
    }
    if(sentSize != buffsize)
        fprintf(stderr, "send(): not all message is sent.");
    close(sockfd);
    return sentSize;
}